﻿//////////////////////////////////////////////
// BlendStateManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class BlendState ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/SingletonClass.h>

// Standards
#include <unordered_map>
#include <mutex>
#include <string>

/// Class ------------------------------------
	
namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT BlendStateManager : public nkCommon::SingletonClass<BlendStateManager>
	{
		friend class nkCommon::SingletonClass<BlendStateManager> ;

		private :

			// Attributs
			// Liste
			std::unordered_map<std::string, BlendState*> _memory ;

			// Default state
			BlendState* _defaultState ;

			// Threading
			std::mutex _mutex ;

		private :

			// Functions
			// Constructeur
			BlendStateManager () ;
		
		public :

			// Destructeur
			~BlendStateManager () ;
		
			// Manipulation mémoire
			BlendState* createOrRetrieve (const std::string_view& name) ;
			BlendState* get (const std::string_view& name) ;
			BlendState* getByIndex (unsigned int index) ;
			void rename (const std::string_view& oldName, const std::string_view& newName) ;
			void erase (const std::string_view& name) ;

			// Getters
			BlendState* getDefaultState () const ;
	} ;
}