﻿//////////////////////////////////////////////
// ImageView.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkImages
{
	class Image ;
}

/// Includes ---------------------------------

// nkImages
#include "../Dll/DllDefines.h"

#include "ImageBase.h"

// nkMemory
#include <NilkinsMemory/Containers/BufferView.h>

/// Class ------------------------------------

namespace nkImages
{
	class DLL_IMAGES_EXPORT ImageView final : public ImageBase
	{
		private :

			// Attributes
			// View over the data, provided from outside
			nkMemory::BufferView<unsigned char> _dataView ;

		public :

			// Constructor, destructor
			ImageView () = delete ;
			ImageView (const Image& image) ;
			ImageView (const nkMemory::BufferView<unsigned char>& dataView) ;
			ImageView (const nkMemory::BufferView<unsigned char> dataView, unsigned int width, unsigned int height, unsigned int depth, PIXEL_FORMAT format, unsigned int pixelOrBlockByteSize = 0, unsigned int rowByteSize = 0, unsigned int sliceByteSize = 0) ;
			ImageView (const nkMemory::BufferView<unsigned char> dataView, unsigned int width, unsigned int height, unsigned int depth, unsigned int arraySize, unsigned int mips, PIXEL_FORMAT format, unsigned int pixelOrBlockByteSize = 0) ;
			ImageView (const ImageView& other) ;
			ImageView (const ImageView& other, unsigned int slice) ;
			ImageView (const ImageView& other, unsigned int slice, unsigned int mip) ;
			ImageView (ImageView&& other) ;
			~ImageView () ;

			// Data access
			virtual unsigned char* getDataPtr () const override ;

			// Operators
			ImageView& operator= (const ImageView& other) ;
			ImageView& operator= (ImageView&& other) ;
	} ;
}