﻿//////////////////////////////////////////////
// Matrix.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkMaths
{
	class Quaternion ;
	class Vector ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/ByteAlignedClass.h>

// Windows / Dx
#include <DirectXMath.h>

// Standards
#include <string_view>

/// Class ------------------------------------

namespace nkMaths
{
	class DLL_MATHS_EXPORT Matrix : public ByteAlignedClass<16>
	{
		private :
		
			// Attributs
			DirectX::XMMATRIX _values ;
		
		public :
		
			// Constructeur, destructeur
			Matrix () ;
			Matrix (float x0, float y0, float z0, float w0, float x1, float y1, float z1, float w1, float x2, float y2, float z2, float w2, float x3, float y3, float z3, float w3) ;
			Matrix (const Matrix& other) ;
			Matrix (const DirectX::XMMATRIX& values) ;
			~Matrix () ;

			// Getters
			DirectX::XMMATRIX getValues () const ;
			Matrix getInverse () const ;
			Matrix getTranspose () const ;
			Vector getPositionComponent () const ;
			Quaternion getOrientationComponent () const ;
			Vector getScaleComponent () const ;
			void getDecomposed (Vector& position, Quaternion& orientation, Vector& scale) ;
			Vector getFirstRow () const ;
			Vector getSecondRow () const ;
			Vector getThirdRow () const ;
			Vector getFourthRow () const ;

			// Setters depuis des vecteurs
			void setToRotationFromQuaternion (const Quaternion& origin) ;

			// Calcul de matrices
			void setToViewMatrixDirection (const Vector& position, const Vector& direction, const Vector& upDirection) ;
			void setToPerspectiveMatrix (float fov, float aspect, float nearVal, float farVal) ;
			void setToOrthographicMatrix (float fov, float camSpaceWidth, float camSpaceHeight, float nearVal, float farVal) ;
			void setToTransformation (const Vector& position, const Quaternion& orientation, const Vector& scale) ;
			void inverse () ;
			void transpose () ;

			// Accès membres
			float get (unsigned int row, unsigned int col) const ;
			void set (float val, unsigned int row, unsigned int col) ;

			// Utilitaires
			float getDeterminantMat3 () const ;
			float getDeterminantMat4 () const ;
			float getTraceMat3 () const ;
			float getTraceMat4 () const ;

			// Pratique
			void fromString (const std::string_view& str) ;

			// Les opérations
			// Assignation
			Matrix& operator= (const Matrix& other) ;
			// Egalité
			bool operator== (const Matrix& other) ;
			bool operator!= (const Matrix& other) ;
			// Addition
			Matrix operator+ (const Matrix& other) const ;
			void operator+= (const Matrix& other) ;
			// Soustraction
			Matrix operator- (const Matrix& other) const ;
			void operator-= (const Matrix& other) ;
			// Multiplications et transfo
			Matrix operator* (const Matrix& other) const ;
			Vector operator* (const Vector& toApply) const ;
			Matrix operator* (float scalar) const ;
			void operator*= (const Matrix& other) ;
			void operator*= (float scalar) ;
			// Division
			Matrix operator/ (float scalar) const ;
			void operator /= (float scalar) ;
	} ;
}