﻿//////////////////////////////////////////////
// ResourceManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkMemory
{
	class Buffer ;
}

/// Includes ---------------------------------

// nkResources
#include "Dll/DllDefines.h"

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// nkMemory
#include <NilkinsMemory/Containers/String.h>
#include <NilkinsMemory/Containers/StringView.h>

/// Class ------------------------------------

namespace nkResources
{
	class DLL_RESOURCES_EXPORT ResourceManager final : public nkCommon::SingletonClass<ResourceManager>
	{
		friend class nkCommon::SingletonClass<ResourceManager> ;

		private :

			// Attributs
			// Le dossier de travail
			nkMemory::String _workingPath ;
			nkMemory::String _workingPathAbsolute ;
			// Le dossier de lancement
			nkMemory::String _launchedPath ;

		private :

			// Functions
			// Constructeur, destructeur
			ResourceManager() ;
			~ResourceManager () ;

		public :

			// Le working path
			void setWorkingPath (const nkMemory::StringView& path) ;
			void setWorkingPathAbsolute (const nkMemory::StringView& path) ;
			nkMemory::StringView getWorkingPath () const ;
			nkMemory::StringView getWorkingPathAbsolute () const ;

			// Path de lancement
			nkMemory::StringView getLaunchedPath () const ;
			void setLaunchedPath (const nkMemory::StringView& path) ;

			// Traduction chemin absolu en relatif
			nkMemory::String getRelativeFromAbsolute (const nkMemory::StringView& absolutePath) const ;
			nkMemory::String getAbsoluteFromRelative (const nkMemory::StringView& relativePath) const ;
			nkMemory::String getRelativeFromWorkingDir (const nkMemory::StringView& workingPath) const ;
			nkMemory::String getAbsoluteFromWorkingDir (const nkMemory::StringView& workingPath) const ;
			nkMemory::String getFromWorkingDirFromRelative (const nkMemory::StringView& relativePath) const ;
			nkMemory::String getFromWorkingDirFromAbsolute (const nkMemory::StringView& absolutePath) const ;

			// Check de folder
			bool checkFileExists (const nkMemory::StringView& filePathAbs) const ;
			void checkFolderExists (const nkMemory::StringView& path) ;

			// Chargement de ressource
			nkMemory::Buffer loadFileIntoMemory (const nkMemory::StringView& filePathAbs, bool silent = false) ;
	} ;
}